@@ -17,6 +17,7 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#include <gio/gsettingsbackend.h>
 #include <glib/gi18n.h>
 #include <string.h>
 
@@ -30,10 +31,9 @@
 
 gchar *get_config_dir_path(void)
 {
-	return g_strconcat(	g_get_user_config_dir(),
-				"/",
-				CONFIG_DIR,
-				NULL );
+	return g_build_filename(	g_get_user_config_dir(),
+					CONFIG_DIR,
+					NULL );
 }
 
 gchar *get_config_file_path(void)
@@ -42,7 +42,7 @@ gchar *get_config_file_path(void)
 	gchar *result;
 
 	config_dir = get_config_dir_path();
-	result = g_strconcat(config_dir, "/" CONFIG_FILE, NULL);
+	result = g_build_filename(config_dir, "gnome-mpv.conf", NULL);
 
 	g_free(config_dir);
 
@@ -99,29 +99,22 @@ gboolean quit(gpointer data)
 	return FALSE;
 }
 
+/* This only supports migrating from v0.5's config file */
 gboolean migrate_config(gmpv_handle *ctx)
 {
-	gchar *config_dir = get_config_dir_path();
 	gchar *config_file = get_config_file_path();
-	gboolean result;
-	char *old_path;
-	GKeyFile *key_config;
-	gchar *keybuf;
+	gchar *backup_config_file = g_strconcat(config_file, ".bak", NULL);
+	gboolean result = FALSE;
 
-	result = FALSE;
-	old_path = g_strconcat(g_get_user_config_dir(), "/", CONFIG_FILE, NULL);
-
-	/* Move config file to the new location if it is at the old location */
-	if(g_file_test(old_path, G_FILE_TEST_EXISTS))
+	if(g_file_test(config_file, G_FILE_TEST_EXISTS))
 	{
-		GFile *src = g_file_new_for_path(old_path);
-		GFile *dest = g_file_new_for_path(config_file);
-
-		g_mkdir_with_parents(config_dir, 0700);
+		/* Backup the old config file */
+		GFile *src = g_file_new_for_path(config_file);
+		GFile *dest = g_file_new_for_path(backup_config_file);
 
 		result = g_file_move(	src,
 					dest,
-					G_FILE_COPY_NONE,
+					G_FILE_COPY_OVERWRITE,
 					NULL,
 					NULL,
 					NULL,
@@ -131,58 +124,54 @@ gboolean migrate_config(gmpv_handle *ctx)
 		g_object_unref(dest);
 	}
 
-	key_config = g_key_file_new();
-
-	g_key_file_load_from_file(	key_config,
-					config_file,
-					G_KEY_FILE_NONE,
-					NULL );
-
-	keybuf = g_key_file_get_string(	key_config,
-					"main",
-					"mpv-options",
-					NULL );
-
-	/* If config file is in the old format, convert it to the new format */
-	if(keybuf && keybuf[0] != '\'' && keybuf[0] != '\"')
+	if(result)
 	{
-		const gchar *opts[] = {	"mpv-options",
-					"mpv-config-file",
-					"mpv-input-config-file",
-					NULL };
+		/* Load settings from the backup file */
+		const gchar *key_list[] = {	"csd-enable",
+						"dark-theme-enable",
+						"mpv-input-config-enable",
+						"mpv-config-enable",
+						"mpv-input-config-file",
+						"mpv-config-file",
+						"mpv-options",
+						NULL };
 
-		const gchar **current = opts;
+		GSettingsBackend *backend;
+		GSettings *keyfile_settings;
+		GSettings *default_settings;
+		const gchar **iter;
 
-		g_free(keybuf);
+		backend = g_keyfile_settings_backend_new
+				(	backup_config_file,
+					"/org/gnome-mpv/gnome-mpv/",
+					"main" );
 
-		while(*current)
+		keyfile_settings = g_settings_new_with_backend(	CONFIG_ROOT,
+								backend );
+
+		default_settings = g_settings_new(CONFIG_ROOT);
+		iter = key_list;
+
+		while(*iter)
 		{
-			keybuf = g_key_file_get_string(	key_config,
-							"main",
-							*current,
-							NULL );
+			GVariant *value;
 
-			g_settings_set_string(	ctx->config,
-						*current,
-						keybuf );
+			value = g_settings_get_value(keyfile_settings, *iter);
 
-			g_free(keybuf);
+			g_settings_set_value(default_settings, *iter, value);
 
-			current++;
+			iter++;
+
+			g_variant_unref(value);
 		}
 
-		keybuf = NULL;
+		g_object_unref(backend);
+		g_object_unref(keyfile_settings);
+		g_object_unref(default_settings);
 	}
 
-	if(keybuf)
-	{
-		g_free(keybuf);
-	}
-
-	g_key_file_free(key_config);
-	g_free(config_dir);
 	g_free(config_file);
-	g_free(old_path);
+	g_free(backup_config_file);
 
 	return result;
 }
@@ -244,7 +233,7 @@ void show_error_dialog(gmpv_handle *ctx, const gchar *prefix, const gchar *msg)
 				GTK_DIALOG_DESTROY_WITH_PARENT,
 				GTK_MESSAGE_ERROR,
 				GTK_BUTTONS_OK,
-				"Error" );
+				_("Error") );
 
 	msg_area = gtk_message_dialog_get_message_area
 			(GTK_MESSAGE_DIALOG(dialog));
@@ -322,7 +311,11 @@ void remove_current_playlist_entry(gmpv_handle *ctx)
 				ctx );
 
 		playlist_widget_remove(playlist, index);
-		mpv_check_error(mpv_command(ctx->mpv_ctx, cmd));
+
+		if(ctx->loaded)
+		{
+			mpv_check_error(mpv_command(ctx->mpv_ctx, cmd));
+		}
 
 		if(playlist_widget_empty(playlist))
 		{
@@ -394,115 +387,6 @@ void toggle_fullscreen(gmpv_handle *ctx)
 				&ctx->gui->fullscreen );
 }
 
-GMenu *build_full_menu()
-{
-	GMenu *menu;
-	GMenu *file_menu;
-	GMenu *edit_menu;
-	GMenu *view_menu;
-	GMenu *help_menu;
-	GMenuItem *file_menu_item;
-	GMenuItem *open_menu_item;
-	GMenuItem *quit_menu_item;
-	GMenuItem *open_loc_menu_item;
-	GMenuItem *save_playlist_menu_item;
-	GMenuItem *edit_menu_item;
-	GMenuItem *load_sub_menu_item;
-	GMenuItem *pref_menu_item;
-	GMenuItem *view_menu_item;
-	GMenuItem *playlist_menu_item;
-	GMenuItem *fullscreen_menu_item;
-	GMenuItem *normal_size_menu_item;
-	GMenuItem *double_size_menu_item;
-	GMenuItem *half_size_menu_item;
-	GMenuItem *help_menu_item;
-	GMenuItem *about_menu_item;
-
-	menu = g_menu_new();
-
-	/* File */
-	file_menu = g_menu_new();
-
-	file_menu_item
-		= g_menu_item_new_submenu
-			(_("_File"), G_MENU_MODEL(file_menu));
-
-	open_menu_item = g_menu_item_new(_("_Open"), "app.open");
-	quit_menu_item = g_menu_item_new(_("_Quit"), "app.quit");
-
-	open_loc_menu_item
-		= g_menu_item_new(_("Open _Location"), "app.openloc");
-
-	save_playlist_menu_item
-		= g_menu_item_new(_("_Save Playlist"), "app.playlist_save");
-
-	/* Edit */
-	edit_menu = g_menu_new();
-
-	edit_menu_item
-		= g_menu_item_new_submenu
-			(_("_Edit"), G_MENU_MODEL(edit_menu));
-
-	load_sub_menu_item
-		= g_menu_item_new(_("_Load Subtitle"), "app.loadsub");
-
-	pref_menu_item
-		= g_menu_item_new(_("_Preferences"), "app.pref");
-
-	/* View */
-	view_menu = g_menu_new();
-
-	view_menu_item
-		= g_menu_item_new_submenu
-			(_("_View"), G_MENU_MODEL(view_menu));
-
-	playlist_menu_item
-		= g_menu_item_new(_("_Toggle Playlist"), "app.playlist_toggle");
-
-	fullscreen_menu_item
-		= g_menu_item_new(_("_Fullscreen"), "app.fullscreen");
-
-	normal_size_menu_item
-		= g_menu_item_new(_("_Normal Size"), "app.normalsize");
-
-	double_size_menu_item
-		= g_menu_item_new(_("_Double Size"), "app.doublesize");
-
-	half_size_menu_item
-		= g_menu_item_new(_("_Half Size"), "app.halfsize");
-
-	/* Help */
-	help_menu = g_menu_new();
-
-	help_menu_item
-		= g_menu_item_new_submenu
-			(_("_Help"), G_MENU_MODEL(help_menu));
-
-	about_menu_item = g_menu_item_new(_("_About"), "app.about");
-
-	g_menu_append_item(menu, file_menu_item);
-	g_menu_append_item(file_menu, open_menu_item);
-	g_menu_append_item(file_menu, open_loc_menu_item);
-	g_menu_append_item(file_menu, save_playlist_menu_item);
-	g_menu_append_item(file_menu, quit_menu_item);
-
-	g_menu_append_item(menu, edit_menu_item);
-	g_menu_append_item(edit_menu, load_sub_menu_item);
-	g_menu_append_item(edit_menu, pref_menu_item);
-
-	g_menu_append_item(menu, view_menu_item);
-	g_menu_append_item(view_menu, playlist_menu_item);
-	g_menu_append_item(view_menu, fullscreen_menu_item);
-	g_menu_append_item(view_menu, normal_size_menu_item);
-	g_menu_append_item(view_menu, double_size_menu_item);
-	g_menu_append_item(view_menu, half_size_menu_item);
-
-	g_menu_append_item(menu, help_menu_item);
-	g_menu_append_item(help_menu, about_menu_item);
-
-	return menu;
-}
-
 void load_keybind(	gmpv_handle *ctx,
 			const gchar *config_path,
 			gboolean notify_propexp )
