@@ -121,6 +121,7 @@ static void control_box_init(ControlBox *box)
 			GTK_STYLE_CLASS_BACKGROUND );
 
 	gtk_range_set_increments(GTK_RANGE(box->seek_bar), 10, 10);
+	gtk_widget_set_sensitive(box->volume_button, FALSE);
 
 	g_object_set(box->play_button, "relief", GTK_RELIEF_NONE, NULL);
 	g_object_set(box->stop_button, "relief", GTK_RELIEF_NONE, NULL);
@@ -238,6 +239,11 @@ void control_box_set_volume(ControlBox *box, gdouble volume)
 		(GTK_SCALE_BUTTON(box->volume_button), volume);
 }
 
+gdouble control_box_get_volume(ControlBox *box)
+{
+	return gtk_scale_button_get_value(GTK_SCALE_BUTTON(box->volume_button));
+}
+
 void control_box_set_playing_state(ControlBox *box, gboolean playing)
 {
 	GtkWidget *play_icon;
@@ -274,5 +280,6 @@ void control_box_reset_control(ControlBox *box)
 {
 	control_box_set_seek_bar_length(box, 0);
 	control_box_set_playing_state(box, FALSE);
+	control_box_set_chapter_enabled(box, FALSE);
 	control_box_set_fullscreen_state(box, FALSE);
 }
