@@ -23,11 +23,8 @@
 #define APP_ID "org.gnome-mpv"
 #define ICON_NAME "gnome-mpv"
 #define CONFIG_DIR "gnome-mpv"
-#define CONFIG_FILE "gnome-mpv.conf"
 #define CONFIG_ROOT APP_ID
 #define CONFIG_WIN_STATE APP_ID".window-state"
-#define CONFIG_ROOT_PATH "/org/gnome-mpv/gnome-mpv/"
-#define CONFIG_ROOT_GROUP "main"
 #define DEFAULT_LOG_LEVEL MPV_LOG_LEVEL_ERROR
 #define MPRIS_TRACK_ID_PREFIX "/org/gnome_mpv/Track/"
 #define MPRIS_BUS_NAME "org.mpris.MediaPlayer2.gnome-mpv"
@@ -52,10 +49,10 @@
 					"< playlist_prev",\
 					"> playlist_next",\
 					"U stop",\
-					"RIGHT seek 10",\
-					"LEFT seek -10",\
-					"UP seek 60",\
-					"DOWN seek -60",\
+					"RIGHT no-osd seek 10",\
+					"LEFT no-osd seek -10",\
+					"UP no-osd seek 60",\
+					"DOWN no-osd seek -60",\
 					"Q quit_watch_later",\
 					"MOUSE_BTN0_DBL cycle fullscreen",\
 					"MOUSE_BTN2 cycle pause",\
