@@ -17,13 +17,13 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <gio/gsettingsbackend.h>
 #include <gio/gio.h>
 #include <glib/gi18n.h>
 #include <gdk/gdk.h>
-#include <epoxy/gl.h>
 #include <locale.h>
 
+#ifdef OPENGL_CB_ENABLED
+#include <epoxy/gl.h>
 #ifdef GDK_WINDOWING_X11
 #include <gdk/gdkx.h>
 #include <epoxy/glx.h>
@@ -36,9 +36,17 @@
 #include <gdk/gdkwin32.h>
 #include <epoxy/wgl.h>
 #endif
+#else
+#ifdef GDK_WINDOWING_X11
+#include <gdk/gdkx.h>
+#else
+#error "X11 GDK backend is required when opengl-cb is disabled."
+#endif
+#endif
 
 #include "def.h"
 #include "common.h"
+#include "menu.h"
 #include "mpv.h"
 #include "keybind.h"
 #include "playlist.h"
@@ -47,8 +55,6 @@
 #include "main_window.h"
 #include "control_box.h"
 #include "playlist_widget.h"
-#include "pref_dialog.h"
-#include "open_loc_dialog.h"
 #include "mpris/mpris.h"
 #include "media_keys/media_keys.h"
 
@@ -85,12 +91,64 @@ static gboolean key_press_handler(	GtkWidget *widget,
 static gboolean mouse_press_handler(	GtkWidget *widget,
 					GdkEvent *event,
 					gpointer data );
+static gboolean get_use_opengl(void);
+static gint64 get_xid(GtkWidget *widget);
+
+#ifdef OPENGL_CB_ENABLED
+static void *get_proc_address(void *fn_ctx, const gchar *name);
 static gboolean vid_area_render_handler(	GtkGLArea *area,
 						GdkGLContext *context,
 						gpointer data );
-static void *get_proc_address(void *fn_ctx, const gchar *name);
-static gboolean get_use_opengl(void);
-static gint64 get_xid(GtkWidget *widget);
+
+static void *get_proc_address(void *fn_ctx, const gchar *name)
+{
+	GdkDisplay *display = gdk_display_get_default();
+
+#ifdef GDK_WINDOWING_WAYLAND
+	if (GDK_IS_WAYLAND_DISPLAY(display))
+		return eglGetProcAddress(name);
+#endif
+#ifdef GDK_WINDOWING_X11
+	if (GDK_IS_X11_DISPLAY(display))
+		return (void *)(intptr_t)glXGetProcAddressARB((const GLubyte *)name);
+#endif
+#ifdef GDK_WINDOWING_WIN32
+	if (GDK_IS_WIN32_DISPLAY(display))
+		return wglGetProcAddress(name);
+#endif
+
+	g_assert_not_reached();
+}
+
+static gboolean vid_area_render_handler(	GtkGLArea *area,
+						GdkGLContext *context,
+						gpointer data )
+{
+	gmpv_handle *ctx = data;
+	int width;
+	int height;
+	int fbo;
+
+	if(!ctx->opengl_ready)
+	{
+		mpv_check_error(mpv_opengl_cb_init_gl(	ctx->opengl_ctx,
+							NULL,
+							get_proc_address,
+							NULL ));
+
+		ctx->opengl_ready = TRUE;
+	}
+
+	width = gtk_widget_get_allocated_width(GTK_WIDGET(area));
+	height = (-1)*gtk_widget_get_allocated_height(GTK_WIDGET(area));
+	fbo = -1;
+
+	glGetIntegerv(GL_FRAMEBUFFER_BINDING, &fbo);
+	mpv_opengl_cb_draw(ctx->opengl_ctx, fbo, width, height);
+
+	return TRUE;
+}
+#endif
 
 static gboolean draw_handler(GtkWidget *widget, cairo_t *cr, gpointer data)
 {
@@ -217,7 +275,7 @@ static gboolean key_press_handler(	GtkWidget *widget,
 	gmpv_handle *ctx = data;
 	guint keyval = ((GdkEventKey*)event)->keyval;
 	guint state = ((GdkEventKey*)event)->state;
-	gchar **command;
+	gchar *command;
 
 	const guint mod_mask =	GDK_MODIFIER_MASK
 				&~(GDK_SHIFT_MASK
@@ -234,7 +292,7 @@ static gboolean key_press_handler(	GtkWidget *widget,
 	/* Try user-specified keys first, then fallback to hard-coded keys */
 	if(command)
 	{
-		mpv_command(ctx->mpv_ctx, (const char **)command);
+		mpv_command_string(ctx->mpv_ctx, command);
 	}
 	else if((state&mod_mask) == 0)
 	{
@@ -268,7 +326,7 @@ static gboolean mouse_press_handler(	GtkWidget *widget,
 {
 	gmpv_handle *ctx = data;
 	GdkEventButton *btn_event = (GdkEventButton *)event;
-	gchar **command;
+	gchar *command;
 
 	command = keybind_get_command(	ctx,
 					TRUE,
@@ -277,7 +335,7 @@ static gboolean mouse_press_handler(	GtkWidget *widget,
 
 	if(command)
 	{
-		mpv_command(ctx->mpv_ctx, (const char **)command);
+		mpv_command_string(ctx->mpv_ctx, command);
 	}
 
 	return TRUE;
@@ -288,34 +346,16 @@ static void app_activate_handler(GApplication *app, gpointer data)
 	gtk_window_present(GTK_WINDOW(((gmpv_handle *)data)->gui));
 }
 
-static void *get_proc_address(void *fn_ctx, const gchar *name)
-{
-	GdkDisplay *display = gdk_display_get_default();
-
-#ifdef GDK_WINDOWING_WAYLAND
-	if (GDK_IS_WAYLAND_DISPLAY(display))
-		return eglGetProcAddress(name);
-#endif
-#ifdef GDK_WINDOWING_X11
-	if (GDK_IS_X11_DISPLAY(display))
-		return (void *)(intptr_t)glXGetProcAddressARB((const GLubyte *)name);
-#endif
-#ifdef GDK_WINDOWING_WIN32
-	if (GDK_IS_WIN32_DISPLAY(display))
-		return wglGetProcAddress(name);
-#endif
-
-	g_assert_not_reached();
-}
-
 static gboolean get_use_opengl(void)
 {
-#ifdef GDK_WINDOWING_X11
+#if defined(OPENGL_CB_ENABLED) && defined(GDK_WINDOWING_X11)
 	/* TODO: Add option to use opengl on X11 */
-	return !GDK_IS_X11_DISPLAY(gdk_display_get_default());
-#else
+	return !GDK_IS_X11_DISPLAY(gdk_display_get_default()) ;
+#elif defined(OPENGL_CB_ENABLED)
 	/* In theory this can work on any backend supporting GtkGLArea */
 	return TRUE;
+#else
+	return FALSE;
 #endif
 }
 
@@ -328,35 +368,6 @@ static gint64 get_xid(GtkWidget *widget)
 #endif
 }
 
-static gboolean vid_area_render_handler(	GtkGLArea *area,
-						GdkGLContext *context,
-						gpointer data )
-{
-	gmpv_handle *ctx = data;
-	int width;
-	int height;
-	int fbo;
-
-	if(!ctx->opengl_ready)
-	{
-		mpv_check_error(mpv_opengl_cb_init_gl(	ctx->opengl_ctx,
-							NULL,
-							get_proc_address,
-							NULL ));
-
-		ctx->opengl_ready = TRUE;
-	}
-
-	width = gtk_widget_get_allocated_width(GTK_WIDGET(area));
-	height = (-1)*gtk_widget_get_allocated_height(GTK_WIDGET(area));
-	fbo = -1;
-
-	glGetIntegerv(GL_FRAMEBUFFER_BINDING, &fbo);
-	mpv_opengl_cb_draw(ctx->opengl_ctx, fbo, width, height);
-
-	return TRUE;
-}
-
 static void app_open_handler(	GApplication *app,
 				gpointer files,
 				gint n_files,
@@ -420,6 +431,7 @@ static void connect_signals(gmpv_handle *ctx)
 
 	playbackctl_connect_signals(ctx);
 
+#ifdef OPENGL_CB_ENABLED
 	if(main_window_get_use_opengl(ctx->gui))
 	{
 		g_signal_connect(	ctx->gui->vid_area,
@@ -427,6 +439,7 @@ static void connect_signals(gmpv_handle *ctx)
 					G_CALLBACK(vid_area_render_handler),
 					ctx );
 	}
+#endif
 
 	g_signal_connect(	ctx->gui->vid_area,
 				"drag-data-received",
@@ -485,7 +498,7 @@ static inline void add_accelerator(	GtkApplication *app,
 
 static void setup_accelerators(gmpv_handle *ctx)
 {
-	add_accelerator(ctx->app, "<Control>o", "app.open");
+	add_accelerator(ctx->app, "<Control>o", "app.open(false)");
 	add_accelerator(ctx->app, "<Control>l", "app.openloc");
 	add_accelerator(ctx->app, "<Control>S", "app.playlist_save");
 	add_accelerator(ctx->app, "<Control>q", "app.quit");
@@ -526,7 +539,6 @@ static void app_startup_handler(GApplication *app, gpointer data)
 {
 	gmpv_handle *ctx = data;
 	const gchar *vid_area_style = ".gmpv-vid-area{background-color: black}";
-	GSettingsBackend *config_backend;
 	GtkCssProvider *style_provider;
 	gboolean css_loaded;
 	gboolean use_opengl;
@@ -534,7 +546,6 @@ static void app_startup_handler(GApplication *app, gpointer data)
 	gboolean mpvinput_enable;
 	gboolean csd_enable;
 	gboolean dark_theme_enable;
-	gchar *config_file;
 	gchar *mpvinput;
 
 	setlocale(LC_NUMERIC, "C");
@@ -545,13 +556,6 @@ static void app_startup_handler(GApplication *app, gpointer data)
 	bind_textdomain_codeset(GETTEXT_PACKAGE, "UTF-8");
 	textdomain(GETTEXT_PACKAGE);
 
-	config_file = get_config_file_path();
-
-	config_backend = g_keyfile_settings_backend_new
-				(	config_file,
-					CONFIG_ROOT_PATH,
-					CONFIG_ROOT_GROUP );
-
 	use_opengl = get_use_opengl();
 
 	ctx->mpv_ctx = mpv_create();
@@ -563,10 +567,11 @@ static void app_startup_handler(GApplication *app, gpointer data)
 	ctx->new_file = TRUE;
 	ctx->sub_visible = TRUE;
 	ctx->init_load = TRUE;
+	ctx->inhibit_cookie = 0;
 	ctx->playlist_move_dest = -1;
 	ctx->log_level_list = NULL;
 	ctx->keybind_list = NULL;
-	ctx->config = g_settings_new_with_backend(APP_ID, config_backend);
+	ctx->config = g_settings_new(CONFIG_ROOT);
 	ctx->app = GTK_APPLICATION(app);
 	ctx->gui = MAIN_WINDOW(main_window_new(ctx->app, use_opengl));
 	ctx->fs_control = NULL;
@@ -611,11 +616,11 @@ static void app_startup_handler(GApplication *app, gpointer data)
 	}
 	else
 	{
-		gtk_application_set_app_menu
-			(ctx->app, NULL);
+		GMenu *full_menu = g_menu_new();
 
-		gtk_application_set_menubar
-			(ctx->app, G_MENU_MODEL(build_full_menu()));
+		menu_build_full(full_menu, NULL, NULL, NULL);
+		gtk_application_set_app_menu (ctx->app, NULL);
+		gtk_application_set_menubar(ctx->app, G_MENU_MODEL(full_menu));
 	}
 
 	gtk_widget_show_all(GTK_WIDGET(ctx->gui));
@@ -657,21 +662,31 @@ static void app_startup_handler(GApplication *app, gpointer data)
 
 	if(config_migrated)
 	{
-		GtkWidget *dialog
-			= gtk_message_dialog_new
+		gchar *config_file;
+		gchar *backup_config_file;
+		GtkWidget *dialog;
+
+		config_file = get_config_file_path();
+		backup_config_file = g_strconcat(config_file, ".bak", NULL);
+
+		dialog = gtk_message_dialog_new
 				(	GTK_WINDOW(ctx->gui),
 					GTK_DIALOG_DESTROY_WITH_PARENT,
 					GTK_MESSAGE_INFO,
 					GTK_BUTTONS_OK,
-					_("Your configuration file has been "
-					"moved to the new location at %s."),
-					config_file );
+					_("Preferences is now stored using "
+					"GSettings. Your preferences have been "
+					"migrated from the configuration file. "
+					"A backup copy of the configuration "
+					"file can be found at \"%s\"."),
+					backup_config_file );
 
 		gtk_dialog_run(GTK_DIALOG(dialog));
 		gtk_widget_destroy(dialog);
+		g_free(config_file);
+		g_free(backup_config_file);
 	}
 
-	g_free(config_file);
 	g_free(mpvinput);
 }
 
@@ -681,6 +696,10 @@ int main(int argc, char **argv)
 	gmpv_handle *ctx;
 	gint status;
 
+#ifndef OPENGL_CB_ENABLED
+	gdk_set_allowed_backends("x11");
+#endif
+
 	app = gtk_application_new(APP_ID, G_APPLICATION_HANDLES_OPEN);
 	ctx = g_malloc(sizeof(gmpv_handle));
 
