@@ -17,11 +17,8 @@
  * along with GNOME MPV.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include <gio/gsettingsbackend.h>
-#include <glib/gi18n.h>
-
 #include "def.h"
-#include "common.h"
+#include "menu.h"
 #include "playlist_widget.h"
 #include "main_window.h"
 #include "control_box.h"
@@ -39,10 +36,9 @@ struct _MainWindowPrivate
 };
 
 static void vid_area_init(MainWindow *wnd, gboolean use_opengl);
+static GtkWidget *vid_area_new(gboolean use_opengl);
 static gboolean timeout_handler(gpointer data);
 static gboolean finalize_load_state(gpointer data);
-static GMenu *menu_btn_build_menu(void);
-static GMenu *open_btn_build_menu(void);
 
 G_DEFINE_TYPE_WITH_PRIVATE(MainWindow, main_window, GTK_TYPE_APPLICATION_WINDOW)
 
@@ -106,8 +102,8 @@ static gboolean motion_notify_handler(GtkWidget *widget, GdkEventMotion *event)
 	MainWindow *wnd = MAIN_WINDOW(widget);
 	GdkCursor *cursor;
 
-	cursor = gdk_cursor_new_for_display(	gdk_display_get_default(),
-						GDK_ARROW );
+	cursor = gdk_cursor_new_from_name(	gdk_display_get_default(),
+						"default" );
 
 	gdk_window_set_cursor
 		(gtk_widget_get_window(GTK_WIDGET(wnd->vid_area)), cursor);
@@ -162,15 +158,18 @@ static void vid_area_init(MainWindow *wnd, gboolean use_opengl)
 	{
 		GtkStyleContext *style_context;
 
-		wnd->vid_area =	use_opengl?
-				gtk_gl_area_new():
-				gtk_drawing_area_new();
-
+		wnd->vid_area =	vid_area_new(use_opengl);
 		style_context = gtk_widget_get_style_context(wnd->vid_area);
 
-		gtk_widget_add_events(wnd->vid_area, GDK_BUTTON_PRESS_MASK);
 		gtk_style_context_add_class(style_context, "gmpv-vid-area");
 
+		/* GDK_BUTTON_RELEASE_MASK is needed so that GtkMenuButtons can
+		 * hide their menus when vid_area is clicked.
+		 */
+		gtk_widget_add_events(	wnd->vid_area,
+					GDK_BUTTON_PRESS_MASK|
+					GDK_BUTTON_RELEASE_MASK );
+
 		gtk_container_add(	GTK_CONTAINER(wnd->vid_area_overlay),
 					wnd->vid_area );
 
@@ -181,6 +180,15 @@ static void vid_area_init(MainWindow *wnd, gboolean use_opengl)
 	}
 }
 
+static GtkWidget *vid_area_new(gboolean use_opengl)
+{
+#ifdef OPENGL_CB_ENABLED
+	return use_opengl?gtk_gl_area_new():gtk_drawing_area_new();
+#else
+	return gtk_drawing_area_new();
+#endif
+}
+
 static gboolean timeout_handler(gpointer data)
 {
 	MainWindow *wnd;
@@ -233,150 +241,53 @@ static gboolean finalize_load_state(gpointer data)
 	return FALSE;
 }
 
-static GMenu *menu_btn_build_menu()
-{
-	GMenu *menu;
-	GMenu *playlist_submenu;
-	GMenu *sub_submenu;
-	GMenu *view_submenu;
-	GMenuItem *playlist_section;
-	GMenuItem *sub_section;
-	GMenuItem *view_section;
-	GMenuItem *load_sub_menu_item;
-	GMenuItem *playlist_toggle_menu_item;
-	GMenuItem *playlist_save_menu_item;
-	GMenuItem *normal_size_menu_item;
-	GMenuItem *double_size_menu_item;
-	GMenuItem *half_size_menu_item;
-
-	menu = g_menu_new();
-	playlist_submenu = g_menu_new();
-	sub_submenu = g_menu_new();
-	view_submenu = g_menu_new();
-
-	playlist_section
-		= g_menu_item_new_section(NULL, G_MENU_MODEL(playlist_submenu));
-
-	sub_section
-		= g_menu_item_new_section(NULL, G_MENU_MODEL(sub_submenu));
-
-	view_section
-		= g_menu_item_new_section(NULL, G_MENU_MODEL(view_submenu));
-
-	playlist_toggle_menu_item
-		= g_menu_item_new(_("_Toggle Playlist"), "app.playlist_toggle");
-
-	playlist_save_menu_item
-		= g_menu_item_new(_("_Save Playlist"), "app.playlist_save");
-
-	load_sub_menu_item
-		= g_menu_item_new(_("_Load Subtitle"), "app.loadsub");
-
-	normal_size_menu_item
-		= g_menu_item_new(_("_Normal Size"), "app.normalsize");
-
-	double_size_menu_item
-		= g_menu_item_new(_("_Double Size"), "app.doublesize");
-
-	half_size_menu_item
-		= g_menu_item_new(_("_Half Size"), "app.halfsize");
-
-	g_menu_append_item(playlist_submenu, playlist_toggle_menu_item);
-	g_menu_append_item(playlist_submenu, playlist_save_menu_item);
-	g_menu_append_item(sub_submenu, load_sub_menu_item);
-	g_menu_append_item(view_submenu, normal_size_menu_item);
-	g_menu_append_item(view_submenu, double_size_menu_item);
-	g_menu_append_item(view_submenu, half_size_menu_item);
-
-	g_menu_append_item(menu, playlist_section);
-	g_menu_append_item(menu, sub_section);
-	g_menu_append_item(menu, view_section);
-
-	return menu;
-}
-
-static GMenu *open_btn_build_menu()
-{
-	GMenu *menu;
-	GMenuItem *open_menu_item;
-	GMenuItem *open_loc_menu_item;
-
-	menu = g_menu_new();
-
-	open_menu_item = g_menu_item_new(_("_Open"), "app.open");
-
-	open_loc_menu_item
-		= g_menu_item_new(_("Open _Location"), "app.openloc");
-
-	g_menu_append_item(menu, open_menu_item);
-	g_menu_append_item(menu, open_loc_menu_item);
-
-	return menu;
-}
-
 void main_window_save_state(MainWindow *wnd)
 {
-	GSettingsBackend *config_backend;
-	GSettings *config;
-	gchar *config_file;
+	GSettings *settings;
 	gint width;
 	gint height;
 	gint handle_pos;
+	gdouble volume;
 
-	config_file = get_config_file_path();
-
-	config_backend = g_keyfile_settings_backend_new
-				(	config_file,
-					CONFIG_ROOT_PATH,
-					CONFIG_ROOT_GROUP );
-
-	config = g_settings_new_with_backend(CONFIG_WIN_STATE, config_backend);
+	settings = g_settings_new(CONFIG_WIN_STATE);
 	handle_pos = gtk_paned_get_position(GTK_PANED(wnd->vid_area_paned));
+	volume = control_box_get_volume(CONTROL_BOX(wnd->control_box));
 
 	gtk_window_get_size(GTK_WINDOW(wnd), &width, &height);
 
-	g_settings_set_int(config, "width", width);
-	g_settings_set_int(config, "height", height);
+	g_settings_set_int(settings, "width", width);
+	g_settings_set_int(settings, "height", height);
+	g_settings_set_double(settings, "volume", volume);
+
+	g_settings_set_boolean(	settings,
+				"show-playlist",
+				wnd->playlist_visible );
 
 	if(main_window_get_playlist_visible(wnd))
 	{
-		g_settings_set_int(	config,
+		g_settings_set_int(	settings,
 					"playlist-width",
 					width-handle_pos );
 	}
 	else
 	{
-		g_settings_set_int(	config,
+		g_settings_set_int(	settings,
 					"playlist-width",
 					wnd->playlist_width );
 	}
 
-	g_settings_set_boolean(	config,
-				"show-playlist",
-				wnd->playlist_visible );
-
-	g_free(config_file);
+	g_clear_object(&settings);
 }
 
 void main_window_load_state(MainWindow *wnd)
 {
-	GSettingsBackend *config_backend;
-	GSettings *config;
-	gchar *config_file;
+	GSettings *settings = g_settings_new(CONFIG_WIN_STATE);
 
-	config_file = get_config_file_path();
-
-	config_backend = g_keyfile_settings_backend_new
-				(	config_file,
-					CONFIG_ROOT_PATH,
-					CONFIG_ROOT_GROUP );
-
-	config = g_settings_new_with_backend(CONFIG_WIN_STATE, config_backend);
-	wnd->init_width = g_settings_get_int(config, "width");
-	wnd->init_height = g_settings_get_int(config, "height");
+	wnd->init_width = g_settings_get_int(settings, "width");
+	wnd->init_height = g_settings_get_int(settings, "height");
 
 	wnd->init_playlist_visible
-		= g_settings_get_boolean(config, "show-playlist");
+		= g_settings_get_boolean(settings, "show-playlist");
 
 	g_signal_connect(	wnd,
 				"configure-event",
@@ -388,11 +299,45 @@ void main_window_load_state(MainWindow *wnd)
 	 */
 	main_window_set_playlist_visible(wnd, TRUE);
 
-	wnd->playlist_width = g_settings_get_int(config, "playlist-width");
+	wnd->playlist_width = g_settings_get_int(settings, "playlist-width");
+
+	control_box_set_volume(	CONTROL_BOX(wnd->control_box),
+				g_settings_get_double(settings, "volume") );
 
 	gtk_window_resize(GTK_WINDOW(wnd), wnd->init_width, wnd->init_height);
 
-	g_free(config_file);
+	g_clear_object(&settings);
+}
+
+void main_window_update_track_list(	MainWindow *wnd,
+					const GSList *audio_list,
+					const GSList *video_list,
+					const GSList *sub_list )
+{
+	if(main_window_get_csd_enabled(wnd))
+	{
+		GMenu *menu = g_menu_new();
+
+		menu_build_menu_btn(menu, audio_list, video_list, sub_list);
+
+		gtk_menu_button_set_menu_model
+			(	GTK_MENU_BUTTON(wnd->menu_hdr_btn),
+				G_MENU_MODEL(menu) );
+	}
+	else
+	{
+		GtkApplication *app;
+		GMenu *menu;
+
+		app = gtk_window_get_application(GTK_WINDOW(wnd));
+		menu = G_MENU(gtk_application_get_menubar(app));
+
+		if(menu)
+		{
+			g_menu_remove_all(menu);
+			menu_build_full(menu, audio_list, video_list, sub_list);
+		}
+	}
 }
 
 static void main_window_class_init(MainWindowClass *klass)
@@ -524,6 +469,9 @@ void main_window_toggle_fullscreen(MainWindow *wnd)
 
 	if(wnd->fullscreen)
 	{
+		GdkCursor *cursor;
+		GdkWindow *vid_area;
+
 		gtk_widget_set_halign(wnd->control_box, GTK_ALIGN_FILL);
 		gtk_widget_set_valign(wnd->control_box, GTK_ALIGN_FILL);
 		gtk_widget_set_size_request(wnd->control_box, -1, -1);
@@ -553,6 +501,13 @@ void main_window_toggle_fullscreen(MainWindow *wnd)
 			gtk_widget_show(wnd->playlist);
 		}
 
+		cursor =	gdk_cursor_new_from_name
+				(gdk_display_get_default(), "default");
+
+		vid_area = gtk_widget_get_window(GTK_WIDGET(wnd->vid_area));
+
+		gdk_window_set_cursor(vid_area, cursor);
+
 		wnd->fullscreen = FALSE;
 	}
 	else
@@ -584,7 +539,6 @@ void main_window_toggle_fullscreen(MainWindow *wnd)
 		gtk_window_fullscreen(GTK_WINDOW(wnd));
 		gtk_window_present(GTK_WINDOW(wnd));
 		gtk_widget_hide(wnd->control_box);
-		timeout_handler(wnd);
 
 		if(main_window_get_csd_enabled(wnd))
 		{
@@ -603,6 +557,7 @@ void main_window_toggle_fullscreen(MainWindow *wnd)
 		}
 
 		wnd->fullscreen = TRUE;
+		timeout_handler(wnd);
 	}
 }
 
@@ -631,10 +586,15 @@ gboolean main_window_get_use_opengl(MainWindow *wnd)
 
 void main_window_enable_csd(MainWindow *wnd)
 {
+	GMenu *menu_btn_menu;
+	GMenu *open_btn_menu;
 	GIcon *open_icon;
 	GIcon *fullscreen_icon;
 	GIcon *menu_icon;
 
+	open_btn_menu = g_menu_new();
+	menu_btn_menu = g_menu_new();
+
 	open_icon = g_themed_icon_new_with_default_fallbacks
 				("list-add-symbolic");
 
@@ -649,6 +609,9 @@ void main_window_enable_csd(MainWindow *wnd)
 	wnd->fullscreen_hdr_btn = gtk_button_new();
 	wnd->menu_hdr_btn = gtk_menu_button_new();
 
+	menu_build_open_btn(open_btn_menu);
+	menu_build_menu_btn(menu_btn_menu, NULL, NULL, NULL);
+
 	gtk_widget_set_can_focus(wnd->open_hdr_btn, FALSE);
 	gtk_widget_set_can_focus(wnd->fullscreen_hdr_btn, FALSE);
 	gtk_widget_set_can_focus(wnd->menu_hdr_btn, FALSE);
@@ -670,11 +633,11 @@ void main_window_enable_csd(MainWindow *wnd)
 
 	gtk_menu_button_set_menu_model
 		(	GTK_MENU_BUTTON(wnd->open_hdr_btn),
-			G_MENU_MODEL(open_btn_build_menu()) );
+			G_MENU_MODEL(open_btn_menu) );
 
 	gtk_menu_button_set_menu_model
 		(	GTK_MENU_BUTTON(wnd->menu_hdr_btn),
-			G_MENU_MODEL(menu_btn_build_menu()) );
+			G_MENU_MODEL(menu_btn_menu) );
 
 	gtk_header_bar_pack_start
 		(GTK_HEADER_BAR(wnd->header_bar), wnd->open_hdr_btn);
