@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2014-2015 gnome-mpv
+ * Copyright (c) 2014-2016 gnome-mpv
  *
  * This file is part of GNOME MPV.
  *
@@ -18,13 +18,13 @@
  */
 
 #include <glib/gi18n.h>
-#include <ctype.h>
 #include <stdlib.h>
 #include <string.h>
 #include <execinfo.h>
 
 #include "mpv.h"
 #include "def.h"
+#include "track.h"
 #include "playlist.h"
 #include "control_box.h"
 #include "playlist_widget.h"
@@ -38,6 +38,8 @@ static void handle_msg_level_opt(gmpv_handle *ctx);
 static void handle_property_change_event(	gmpv_handle *ctx,
 						mpv_event_property* prop);
 static void opengl_callback(void *cb_ctx);
+static void uninit_opengl_cb(gmpv_handle *ctx);
+static Track *parse_track_list(mpv_node_list *node);
 
 static void parse_dim_string(	gmpv_handle *ctx,
 				const gchar *mpv_geom_str,
@@ -280,42 +282,76 @@ static void handle_property_change_event(	gmpv_handle *ctx,
 {
 	if(g_strcmp0(prop->name, "pause") == 0)
 	{
+		GtkWindow *wnd = GTK_WINDOW(ctx->gui);
+		GtkApplication *app = gtk_window_get_application(wnd);
+		gboolean idle;
+
 		ctx->paused = prop->data?*((int *)prop->data):TRUE;
 
-		if(!ctx->loaded && !ctx->paused)
+		mpv_get_property(ctx->mpv_ctx, "idle", MPV_FORMAT_FLAG, &idle);
+
+		if(idle && !ctx->paused)
 		{
 			mpv_load(ctx, NULL, FALSE, TRUE);
 		}
 
+		if(!ctx->paused)
+		{
+			ctx->inhibit_cookie
+				= gtk_application_inhibit
+					(	app, wnd,
+						GTK_APPLICATION_INHIBIT_IDLE,
+						_("Playing") );
+		}
+		else if(ctx->inhibit_cookie != 0)
+		{
+			gtk_application_uninhibit(app, ctx->inhibit_cookie);
+		}
+
 		mpv_load_gui_update(ctx);
 	}
-	else if(g_strcmp0(prop->name, "volume") == 0)
+	else if(g_strcmp0(prop->name, "volume") == 0
+	&& (ctx->init_load || ctx->loaded))
 	{
-		ControlBox *control_box;
-		gdouble volume;
+		gchar *aid  = mpv_get_property_string(ctx->mpv_ctx, "aid");
+		ControlBox *control_box = CONTROL_BOX(ctx->gui->control_box);
 
-		control_box = CONTROL_BOX(ctx->gui->control_box);
-		volume = prop->data?*((double *)prop->data)/100.0:0;
+		if(g_strcmp0(aid, "no") == 0)
+		{
+			gtk_widget_set_sensitive
+				(control_box->volume_button, FALSE);
+		}
+		else
+		{
+			gdouble volume;
 
-		g_signal_handlers_block_matched
-			(	control_box->volume_button,
-				G_SIGNAL_MATCH_DATA,
-				0,
-				0,
-				NULL,
-				NULL,
-				ctx );
+			volume = prop->data?*((double *)prop->data)/100.0:0;
 
-		control_box_set_volume(control_box, volume);
+			gtk_widget_set_sensitive
+				(control_box->volume_button, TRUE);
 
-		g_signal_handlers_unblock_matched
-			(	control_box->volume_button,
-				G_SIGNAL_MATCH_DATA,
-				0,
-				0,
-				NULL,
-				NULL,
-				ctx );
+			g_signal_handlers_block_matched
+				(	control_box->volume_button,
+					G_SIGNAL_MATCH_DATA,
+					0,
+					0,
+					NULL,
+					NULL,
+					ctx );
+
+			control_box_set_volume(control_box, volume);
+
+			g_signal_handlers_unblock_matched
+				(	control_box->volume_button,
+					G_SIGNAL_MATCH_DATA,
+					0,
+					0,
+					NULL,
+					NULL,
+					ctx );
+		}
+
+		mpv_free(aid);
 	}
 	else if(g_strcmp0(prop->name, "fullscreen") == 0)
 	{
@@ -341,12 +377,62 @@ static void handle_property_change_event(	gmpv_handle *ctx,
 
 static void opengl_callback(void *cb_ctx)
 {
+#ifdef OPENGL_CB_ENABLED
 	gmpv_handle *ctx = cb_ctx;
 
 	if(ctx->opengl_ctx)
 	{
 		gtk_gl_area_queue_render(GTK_GL_AREA(ctx->gui->vid_area));
 	}
+#endif
+}
+
+static void uninit_opengl_cb(gmpv_handle *ctx)
+{
+#ifdef OPENGL_CB_ENABLED
+	gtk_gl_area_make_current(GTK_GL_AREA(ctx->gui->vid_area));
+	mpv_opengl_cb_uninit_gl(ctx->opengl_ctx);
+#endif
+}
+
+static Track *parse_track_list(mpv_node_list *node)
+{
+	Track *entry = track_new();
+
+	for(gint i = 0; i < node->num; i++)
+	{
+		if(g_strcmp0(node->keys[i], "type") == 0)
+		{
+			const gchar *type = node->values[i].u.string;
+
+			if(g_strcmp0(type, "audio") == 0)
+			{
+				entry->type = TRACK_TYPE_AUDIO;
+			}
+			else if(g_strcmp0(type, "video") == 0)
+			{
+				entry->type = TRACK_TYPE_VIDEO;
+			}
+			else if(g_strcmp0(type, "sub") == 0)
+			{
+				entry->type = TRACK_TYPE_SUBTITLE;
+			}
+		}
+		else if(g_strcmp0(node->keys[i], "title") == 0)
+		{
+			entry->title = g_strdup(node->values[i].u.string);
+		}
+		else if(g_strcmp0(node->keys[i], "lang") == 0)
+		{
+			entry->lang = g_strdup(node->values[i].u.string);
+		}
+		else if(g_strcmp0(node->keys[i], "id") == 0)
+		{
+			entry->id = node->values[i].u.int64;
+		}
+	}
+
+	return entry;
 }
 
 void mpv_wakeup_callback(void *data)
@@ -395,7 +481,26 @@ void mpv_log_handler(gmpv_handle *ctx, mpv_event_log_message* message)
 
 	if(!iter || (message->log_level <= level->level))
 	{
-		show_error_dialog(ctx, message->prefix, message->text);
+		gchar *buf = g_strdup(message->text);
+		gsize len = strlen(buf);
+
+		if(len > 1)
+		{
+			/* g_message() automatically adds a newline
+			 * character when using the default log handler,
+			 * but log messages from mpv already come
+			 * terminated with a newline character so we
+			 * need to take it out.
+			 */
+			if(buf[len-1] == '\n')
+			{
+				buf[len-1] = '\0';
+			}
+
+			g_message("[%s] %s", message->prefix, buf);
+		}
+
+		g_free(buf);
 	}
 }
 
@@ -440,8 +545,6 @@ gboolean mpv_handle_event(gpointer data)
 		{
 			if(ctx->init_load)
 			{
-				ctx->init_load = FALSE;
-
 				mpv_load(ctx, NULL, FALSE, FALSE);
 			}
 			else if(ctx->loaded)
@@ -460,20 +563,44 @@ gboolean mpv_handle_event(gpointer data)
 				main_window_reset(ctx->gui);
 				playlist_reset(ctx);
 			}
+
+			ctx->init_load = FALSE;
 		}
 		else if(event->event_id == MPV_EVENT_FILE_LOADED)
 		{
 			ctx->loaded = TRUE;
+			ctx->init_load = FALSE;
 
 			mpv_update_playlist(ctx);
 			mpv_load_gui_update(ctx);
 		}
 		else if(event->event_id == MPV_EVENT_END_FILE)
 		{
+			mpv_event_end_file *ef_event = event->data;
+
+			ctx->init_load = FALSE;
+
 			if(ctx->loaded)
 			{
 				ctx->new_file = FALSE;
 			}
+
+			if(ef_event->reason == MPV_END_FILE_REASON_ERROR)
+			{
+				const gchar *err_str;
+				gchar *msg;
+
+				err_str = mpv_error_string(ef_event->error);
+
+				msg = g_strdup_printf
+					(	_("Playback was terminated "
+						"abnormally. Reason: %s."),
+						err_str );
+
+				show_error_dialog(ctx, NULL, msg);
+
+				g_free(msg);
+			}
 		}
 		else if(event->event_id == MPV_EVENT_VIDEO_RECONFIG)
 		{
@@ -515,20 +642,25 @@ void mpv_update_playlist(gmpv_handle *ctx)
 	 */
 	const gsize filename_prop_str_size = 38;
 	PlaylistWidget *playlist;
-	mpv_node playlist_array;
+	GtkListStore *store;
+	GtkTreeIter iter;
+	mpv_node mpv_playlist;
 	gchar *filename_prop_str;
+	gboolean iter_end;
 	gint playlist_count;
 	gint i;
 
 	playlist = PLAYLIST_WIDGET(ctx->gui->playlist);
+	store = playlist->list_store;
 	filename_prop_str = g_malloc(filename_prop_str_size);
+	iter_end = FALSE;
 
 	mpv_check_error(mpv_get_property(	ctx->mpv_ctx,
 						"playlist",
 						MPV_FORMAT_NODE,
-						&playlist_array ));
+						&mpv_playlist ));
 
-	playlist_count = playlist_array.u.list->num;
+	playlist_count = mpv_playlist.u.list->num;
 
 	g_signal_handlers_block_matched
 		(	playlist->list_store,
@@ -539,41 +671,83 @@ void mpv_update_playlist(gmpv_handle *ctx)
 			NULL,
 			ctx );
 
-	playlist_widget_clear(playlist);
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(store), &iter);
 
 	for(i = 0; i < playlist_count; i++)
 	{
 		gint prop_count = 0;
-		gchar *path = NULL;
+		gchar *uri = NULL;
 		gchar *title = NULL;
 		gchar *name = NULL;
+		gchar *old_name = NULL;
+		gchar *old_uri = NULL;
 
-		prop_count = playlist_array.u.list->values[i].u.list->num;
+		prop_count = mpv_playlist.u.list->values[i].u.list->num;
 
 		/* The first entry must always exist */
-		path =	playlist_array.u.list
+		uri =	mpv_playlist.u.list
 			->values[i].u.list
 			->values[0].u.string;
 
 		/* Try retrieving the title from mpv playlist */
 		if(prop_count >= 4)
 		{
-			title = playlist_array.u.list
+			title = mpv_playlist.u.list
 				->values[i].u.list
 				->values[3].u.string;
 		}
 
-		if(!title)
+		name = title?title:get_name_from_path(uri);
+
+		if(!iter_end)
 		{
-			name = get_name_from_path(path);
+			gtk_tree_model_get
+				(	GTK_TREE_MODEL(store), &iter,
+					PLAYLIST_NAME_COLUMN, &old_name,
+					PLAYLIST_URI_COLUMN, &old_uri, -1 );
+
+			if(g_strcmp0(name, old_name) != 0)
+			{
+				gtk_list_store_set
+					(	store, &iter,
+						PLAYLIST_NAME_COLUMN, name, -1 );
+			}
+
+			if(g_strcmp0(uri, old_uri) != 0)
+			{
+				gtk_list_store_set
+					(	store, &iter,
+						PLAYLIST_URI_COLUMN, uri, -1 );
+			}
+
+			iter_end = !gtk_tree_model_iter_next
+					(GTK_TREE_MODEL(store), &iter);
+
+			g_free(old_name);
+			g_free(old_uri);
+		}
+		/* Append entries to the playlist if there are fewer entries in
+		 * the playlist widget than mpv's playlist.
+		 */
+		else
+		{
+			playlist_widget_append(playlist, name, uri);
 		}
 
-		playlist_widget_append(playlist, title?title:name, path);
-
-		mpv_free(path);
+		mpv_free(uri);
 		g_free(name);
 	}
 
+	/* If there are more entries in the playlist widget than mpv's playlist,
+	 * remove the excess entries from the playlist widget.
+	 */
+	if(!iter_end)
+	{
+		while(gtk_list_store_remove(store, &iter));
+	}
+
+	gtk_tree_view_columns_autosize(GTK_TREE_VIEW(playlist->tree_view));
+
 	g_signal_handlers_unblock_matched
 		(	playlist->list_store,
 			G_SIGNAL_MATCH_DATA,
@@ -584,12 +758,13 @@ void mpv_update_playlist(gmpv_handle *ctx)
 			ctx );
 
 	g_free(filename_prop_str);
-	mpv_free_node_contents(&playlist_array);
+	mpv_free_node_contents(&mpv_playlist);
 }
 
 void mpv_load_gui_update(gmpv_handle *ctx)
 {
 	ControlBox *control_box;
+	mpv_node track_list;
 	gchar* title;
 	gint64 chapter_count;
 	gint64 playlist_pos;
@@ -611,6 +786,86 @@ void mpv_load_gui_update(gmpv_handle *ctx)
 						MPV_FORMAT_FLAG,
 						&ctx->paused));
 
+	if(mpv_get_property(	ctx->mpv_ctx,
+				"track-list",
+				MPV_FORMAT_NODE,
+				&track_list) >= 0)
+	{
+		mpv_node_list *org_list = track_list.u.list;
+		GSList *audio_list = NULL;
+		GSList *video_list = NULL;
+		GSList *sub_list = NULL;
+		GAction *action = NULL;
+		gint64 aid = -1;
+		gint64 sid = -1;
+
+		mpv_get_property(	ctx->mpv_ctx,
+					"aid",
+					MPV_FORMAT_INT64,
+					&aid );
+
+		mpv_get_property(	ctx->mpv_ctx,
+					"sid",
+					MPV_FORMAT_INT64,
+					&sid );
+
+		action = g_action_map_lookup_action
+				(G_ACTION_MAP(ctx->app), "audio_select");
+
+		g_simple_action_set_state
+			(G_SIMPLE_ACTION(action), g_variant_new_int64(aid));
+
+		action = g_action_map_lookup_action
+				(G_ACTION_MAP(ctx->app), "sub_select");
+
+		g_simple_action_set_state
+			(G_SIMPLE_ACTION(action), g_variant_new_int64(sid));
+
+		for(gint i = 0; i < org_list->num; i++)
+		{
+			Track *entry;
+			GSList **list;
+
+			entry = parse_track_list
+				(org_list->values[i].u.list);
+
+			if(entry->type == TRACK_TYPE_AUDIO)
+			{
+				list = &audio_list;
+			}
+			else if(entry->type == TRACK_TYPE_VIDEO)
+			{
+				list = &video_list;
+			}
+			else if(entry->type == TRACK_TYPE_SUBTITLE)
+			{
+				list = &sub_list;
+			}
+			else
+			{
+				g_assert(FALSE);
+			}
+
+			*list = g_slist_prepend(*list, entry);
+		}
+
+		audio_list = g_slist_reverse(audio_list);
+		video_list = g_slist_reverse(video_list);
+		sub_list = g_slist_reverse(sub_list);
+
+		main_window_update_track_list
+			(ctx->gui, audio_list, video_list, sub_list);
+
+		g_slist_free_full
+			(audio_list, (GDestroyNotify)track_free);
+
+		g_slist_free_full
+			(video_list, (GDestroyNotify)track_free);
+
+		g_slist_free_full
+			(sub_list, (GDestroyNotify)track_free);
+	}
+
 	if(mpv_get_property(	ctx->mpv_ctx,
 				"playlist-pos",
 				MPV_FORMAT_INT64,
@@ -697,8 +952,12 @@ gint mpv_apply_args(mpv_handle *mpv_ctx, gchar *args)
 			token_arg = "";
 		}
 
+		/* Ignore --autofit since it triggers weird behavior in mpv when
+		 * running on Wayland and opengl-cb is disabled
+		 */
+		if(g_strcmp0(token, "autofit") != 0
 		/* Failing to apply extra options is non-fatal */
-		if(mpv_set_option_string(mpv_ctx, token, token_arg) < 0)
+		&& mpv_set_option_string(mpv_ctx, token, token_arg) < 0)
 		{
 			fail_count++;
 
@@ -722,14 +981,12 @@ gint mpv_apply_args(mpv_handle *mpv_ctx, gchar *args)
 
 void mpv_init(gmpv_handle *ctx)
 {
+	GSettings *settings = g_settings_new(CONFIG_WIN_STATE);
+	gdouble volume = g_settings_get_double(settings, "volume")*100;
 	gboolean mpvconf_enable = FALSE;
 	gchar *config_dir = get_config_dir_path();
 	gchar *mpvconf = NULL;
 	gchar *mpvopt = NULL;
-	gchar *screenshot_template = NULL;
-
-	screenshot_template
-		= g_build_filename(g_get_home_dir(), "screenshot-%n", NULL);
 
 	/* Set default options */
 	mpv_check_error(mpv_set_option_string(ctx->mpv_ctx, "osd-level", "1"));
@@ -747,6 +1004,10 @@ void mpv_init(gmpv_handle *ctx)
 						"title",
 						"${media-title}" ));
 
+	mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
+						"pause",
+						"yes" ));
+
 	mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
 						"ytdl",
 						"yes" ));
@@ -767,13 +1028,18 @@ void mpv_init(gmpv_handle *ctx)
 						"config",
 						"yes" ));
 
+	mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
+						"screenshot-template",
+						"gnome-mpv-shot%n" ));
+
 	mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
 						"config-dir",
 						config_dir ));
 
-	mpv_check_error(mpv_set_option_string(	ctx->mpv_ctx,
-						"screenshot-template",
-						screenshot_template ));
+	mpv_check_error(mpv_set_option(	ctx->mpv_ctx,
+					"volume",
+					MPV_FORMAT_DOUBLE,
+					&volume ));
 
 	if(!main_window_get_use_opengl(ctx->gui))
 	{
@@ -841,10 +1107,10 @@ void mpv_init(gmpv_handle *ctx)
 						opengl_callback,
 						ctx );
 
+	g_clear_object(&settings);
 	g_free(config_dir);
 	g_free(mpvconf);
 	g_free(mpvopt);
-	g_free(screenshot_template);
 }
 
 void mpv_quit(gmpv_handle *ctx)
@@ -852,8 +1118,7 @@ void mpv_quit(gmpv_handle *ctx)
 	if(gtk_widget_get_realized(ctx->gui->vid_area)
 	&& main_window_get_use_opengl(ctx->gui))
 	{
-		gtk_gl_area_make_current(GTK_GL_AREA(ctx->gui->vid_area));
-		mpv_opengl_cb_uninit_gl(ctx->opengl_ctx);
+		uninit_opengl_cb(ctx);
 
 		ctx->opengl_ready = FALSE;
 	}
